from docstring_parser.google import Section, SectionType

DEFAULT_SECTIONS = [
    Section("Arguments", "param", SectionType.MULTIPLE),
    Section("Args", "param", SectionType.MULTIPLE),
    Section("Labels", "label", SectionType.MULTIPLE),
    Section("Parameters", "param", SectionType.MULTIPLE),
    Section("Params", "param", SectionType.MULTIPLE),
    Section("Raises", "raises", SectionType.MULTIPLE),
    Section("Exceptions", "raises", SectionType.MULTIPLE),
    Section("Except", "raises", SectionType.MULTIPLE),
    Section("Attributes", "attribute", SectionType.MULTIPLE),
    Section("Example", "examples", SectionType.SINGULAR_OR_MULTIPLE),
    Section("Example 1", "examples", SectionType.SINGULAR_OR_MULTIPLE),
    Section("Example 2", "examples", SectionType.SINGULAR_OR_MULTIPLE),
    Section("Example 3", "examples", SectionType.SINGULAR_OR_MULTIPLE),
    Section("Example 4", "examples", SectionType.SINGULAR_OR_MULTIPLE),
    Section("Example 5", "examples", SectionType.SINGULAR_OR_MULTIPLE),
    Section("Examples", "examples", SectionType.SINGULAR_OR_MULTIPLE),
    Section("Author", "author", SectionType.SINGULAR),
    Section("Returns", "returns", SectionType.SINGULAR_OR_MULTIPLE),
    Section("Yields", "yields", SectionType.SINGULAR_OR_MULTIPLE),
    Section("Input Concepts", "inputs", SectionType.MULTIPLE),
    Section("Output Concepts", "outputs", SectionType.MULTIPLE),
    Section("OAuth Arguments", "param", SectionType.MULTIPLE),
    Section("OAuth Labels", "label", SectionType.MULTIPLE),
]
